
class Vector3d {
  final double x, y, z;
  
  public Vector3d(double x, double y, double z) {
    this.x = x;
    this.y = y;
    this.z = z;
  }
  
  public Vector3d normalize() {
    double sc = 1 / norm();
    return new Vector3d(x*sc, y*sc, z*sc);
  }
  
  public Vector3d plus(Vector3d that) {
    return new Vector3d(x + that.x, y + that.y, z + that.z);
  }
  
  public Vector3d times(double a) {
    return new Vector3d(a * x, a * y, a*z);
  }
  
  public Vector3d minus(Vector3d that) {
    return this.plus(that.times(-1));
  }
  
  public double normSq() {
    return x*x + y*y;
  }
  
  public double norm() {
    return Math.sqrt(normSq());
  }
  
  public double dot(Vector3d that) {
    return x*that.x + y*that.y + z*that.z;
  }  
  
  public double cosThetaTo(Vector3d that) {
    return this.dot(that) / (norm() * that.norm());
  }
  
  public double thetaTo(Vector3d that) {
    return Math.acos(cosThetaTo(that));
  }
}

// Based on : http://stackoverflow.com/questions/2931573/determining-if-two-rays-intersect
/**
 * Returns the point at which a ray in diretion v1 from point O1 hits the ray in direction v2 from point O2. 
 */ 
Point3d rayIntersection(Point3d O1, Vector3d v1, Point3d O2, Vector3d v2) {
  double dx = O2.x - O1.x;
  double dy = O2.y - O1.y;
  double det = v2.x * v1.y - v2.y * v1.x;
  double u = (dy * v2.x - dx * v2.y) / det;
  Vector3d scale = v1.times(u);
  return new Point3d(O1.x + scale.x, O1.y + scale.y, 0);
}